@@ -36,6 +36,36 @@ class StoredEntity(Entity):
         if self.queue:
             self.log('w', 'commit')
 
+    @_init_obj_wrapper
+    def list_parts(self):
+        return self.obj.list_parts()
+        
+    @_init_obj_wrapper
+    def list_part_versions(self, part_id):
+        return self.obj.list_part_versions(part_id)
+        
+    @_init_obj_wrapper
+    def del_part_version(self, part_id, version):
+        r = self.obj.del_part_version(part_id, version)
+        if self.queue:
+            self.log('d','version', part_id=part_id, version=version)
+        return r
+
+    @_init_obj_wrapper
+    def del_part_versions(self, part_id, versions, force=False):
+        r = self.obj.del_part_versions(part_id, versions, force)
+        if self.queue:
+            self.log('d','versions', part_id=part_id, versions=versions)
+        return r
+
+    @_init_obj_wrapper
+    def remove_previous_versions(self, part_id):
+        versions = self.obj.list_part_versions(part_id)
+        versions.sort()
+        most_current = versions.pop()
+        self.del_part_versions(part_id, versions)
+        return most_current
+
     @_init_obj_wrapper
     def revert(self, create_if_nonexistent=True):
         self.load_root(create_if_nonexistent)
@@ -43,7 +73,9 @@ class StoredEntity(Entity):
 
     @_init_obj_wrapper
     def store_root(self):
-        self.obj.putRoot(self.root)
+        # only store the root if it has changed:
+        if self.root.altered:
+            self.obj.putRoot(self.root)
         if self.queue:
             self.log('w','root')
 
@@ -52,18 +84,41 @@ class StoredEntity(Entity):
         self.root = self.obj.getRoot()
 
     @_init_obj_wrapper
-    def put_stream(self, stream_id, bytestream, buffer_size=1024*8):
-        # TODO: automatically update the manifest
-        self.obj.add_part(stream_id, bytestream, buffer_size)
+    def put_stream(self, part_id, bytestream):
+        self.obj.add_part(part_id, bytestream)
+        part_uri = self.uh.parse_uri("%s/%s" % (self.uri, part_id))
+        self.add_triple(self.uri, "ore:aggregates", part_uri)
+        self.add_triple(part_uri, "dcterms:isPartOf", self.uri)
+        self.commit()
         if self.queue:
-            self.log('w','stream', stream_id=stream_id)
+            self.log('w','stream', part_id=part_id)
 
     @_init_obj_wrapper
-    def get_stream(self, stream_id):
-        # TODO: automatically update the manifest
-        stream = self.obj.get_part(stream_id, stream=True)
+    def del_stream(self, part_id):
+        self.obj.del_part(part_id)
+        part_uri = self.uh.parse_uri("%s/%s" % (self.uri, part_id))
+        self.del_triple(self.uri, "ore:aggregates", part_uri)
+        try:
+            # Try to remove any triples where this part was the subject
+            self.manifest.del_item(part_uri)
+        except:
+            pass
+        self.commit()
         if self.queue:
-            self.log('r','stream', stream_id=stream_id)
+            self.log('d','stream', part_id=part_id)
+
+    @_init_obj_wrapper
+    def get_stream(self, part_id, version=None):
+        stream = self.obj.get_part(part_id, stream=True, version=version)
+        if self.queue:
+            self.log('r','stream', part_id=part_id)
+        return stream
+
+    @_init_obj_wrapper
+    def get_as_bytes(self, part_id, version=None):
+        stream = self.obj.get_part(part_id, stream=False, version=version)
+        if self.queue:
+            self.log('r','stream', part_id=part_id)
         return stream
 
     @_init_obj_wrapper
@@ -72,6 +127,7 @@ class StoredEntity(Entity):
             if part_uri.startswith(self.obj.uri) and self.parts_objs[part_uri].altered:
                 part_id = part_uri[len(self.obj.uri)+1:]
                 self.obj.add_part(part_id, self.parts_objs[part_uri].to_string())
+                self.add_triple(self.uri, "ore:aggregates", part_uri)
                 if self.queue:
                     self.log('w','part', part_id=part_id)
 
@@ -107,6 +163,7 @@ class StoredEntity(Entity):
         if graph_uri not in self.parts:
             g = super(StoredEntity, self).add_named_graph(graph_id, valid_from=valid_from, valid_until=valid_until)
             self.obj.add_part(graph_id, g.to_string())
+            self.commit()
             if self.queue:
                 self.log('w','namedgraph', namedgraph_id=graph_id)
             return g
@@ -119,6 +176,7 @@ class StoredEntity(Entity):
     def del_named_graph(self, graph_id):
         super(StoredEntity, self).del_named_graph(graph_id)
         self.obj.del_part(graph_id)
+        self.commit()
         if self.queue:
             self.log('d','namedgraph', namedgraph_id=graph_id)
 
