@@ -14,20 +14,14 @@ else:
 
 from rdfobject import FileMultiEntityFactory
 
+from rdfobject.stores.storage_exceptions import *
+
+from rdfobject.stores import StoreAlreadyExistsException
+
+
 f = FileMultiEntityFactory()
 
 print f.status
-
-if 'fileentitytest' in f.keys():
-    ora1 = f['fileentitytest'].get('info:fedora/ora:1')
-    ora1.revert()
-    print ora1.uri
-
-    ora1.add_triple(ora1.uri, 'dc:name', 'Umbongo')
-    print dir(ora1)
-    print ora1.root.to_string()
-    ora1.commit()
-
 foo = f.add_store('foo', 'info:media/')
 
 vid = foo.get('info:media/video:1')
@@ -36,7 +30,32 @@ vid.add_triple(vid.uri, 'dc:title', 'My Video Diary')
 vid.add_triple(vid.uri, 'dc:description', 'Boring video')
 vid.commit()
 
-bar = f.clone_store('foo', 'barbar')
+vid.put_stream('foobar', """Blah blah blah""")
+vid.put_stream('foobar', """Blah blah blah (v2)""")
+
+vid.put_stream('deleteme', """Blah blah blah""")
+vid.put_stream('deleteme', """Blah blah blah""")
+vid.put_stream('deleteme', """Blah blah blah""")
+
+# version delete
+
+try:
+    vid.del_part_version('deleteme', 1)
+except VersionNotFoundException:
+    print "delete me version 1 not found"
+
+# part delete
+
+vid.del_stream('deleteme')
+
+print vid
+
+print "Parts: %s" % vid.list_parts()
+
+print "Versions of ROOT: %s" % vid.list_part_versions('ROOT')
+try:
+    bar = f.clone_store('foo', 'barbar')
+except StoreAlreadyExistsException:
+    print "Store barbar already exists."
 
-f.delete_store('foo')
 
